from gstools import SRF, Exponential
from matplotlib.pyplot import figure, imshow, show, xlabel, ylabel
from numpy import arange, mean, ones, power, size, var, zeros
from pickle import dump
from scipy.signal import convolve

cells_per_unit = int(10)
step_size = float(1.0/cells_per_unit)

def gen_convolver(convolver_size):
    convolver_steps = int(float(convolver_size)*cells_per_unit)
    convolver = ones((convolver_steps,convolver_steps))/convolver_steps**2
    return(convolver)

log_mean_K = -2

corr_len = float(0.5)
width_x = float(20)
width_y = float(20)

range_x = arange(0, width_x, step_size)
range_y = arange(0, width_y, step_size)

model = Exponential(dim=2,var=0.5,len_scale=corr_len)

srf = SRF(model, seed=42)
field = srf((range_x,range_y), mesh_type='structured')

field_basic = convolve(field, gen_convolver(0.8), mode='same') + log_mean_K
field_smoothed = convolve(field, gen_convolver(1.6) ,mode='same') + log_mean_K

print(mean(field_basic))
print(mean(field_smoothed))
print(var(field_basic))
print(var(field_smoothed))

figure()
imshow(field_basic, origin='lower', extent=[0, width_x, 0, width_y])
xlabel('x [m]')
ylabel('y [m]')

figure()
imshow(field_smoothed, origin='lower', extent=[0, width_x, 0, width_y])
xlabel('x [m]')
ylabel('y [m]')

show()

fields = (field_basic,field_smoothed)
names = ('k_field_basic.pickle','k_field_smoothed.pickle')

for field_idx in range(len(fields)):
    dump_data = {
        'field' : power(10,fields[field_idx]),
        'width_x' : width_x,
        'width_y' : width_y
    }
    k_filename = names[field_idx]

    pickle_file = open(k_filename,mode='wb')
    dump(dump_data, pickle_file)
    pickle_file.close()
